@@ -8,7 +8,6 @@
 
 //! The data (inner representation) traits for ndarray
 
-use crate::extension::nonnull::nonnull_from_vec_data;
 use rawpointer::PointerExt;
 use std::mem::{self, size_of};
 use std::ptr::NonNull;
@@ -188,7 +187,7 @@ unsafe impl<A> RawDataClone for RawViewRepr<*mut A> {
 unsafe impl<A> RawData for OwnedArcRepr<A> {
     type Elem = A;
     fn _data_slice(&self) -> Option<&[A]> {
-        Some(&self.0)
+        Some(self.0.as_slice())
     }
     private_impl! {}
 }
@@ -226,7 +225,7 @@ where
         };
         let rvec = Arc::make_mut(rcvec);
         unsafe {
-            self_.ptr = nonnull_from_vec_data(rvec).offset(our_off);
+            self_.ptr = rvec.as_nonnull_mut().offset(our_off);
         }
     }
 
@@ -242,7 +241,7 @@ unsafe impl<A> Data for OwnedArcRepr<A> {
         D: Dimension,
     {
         Self::ensure_unique(&mut self_);
-        let data = OwnedRepr(Arc::try_unwrap(self_.data.0).ok().unwrap());
+        let data = Arc::try_unwrap(self_.data.0).ok().unwrap();
         ArrayBase {
             data,
             ptr: self_.ptr,
@@ -264,7 +263,7 @@ unsafe impl<A> RawDataClone for OwnedArcRepr<A> {
 unsafe impl<A> RawData for OwnedRepr<A> {
     type Elem = A;
     fn _data_slice(&self) -> Option<&[A]> {
-        Some(&self.0)
+        Some(self.as_slice())
     }
     private_impl! {}
 }
@@ -303,10 +302,10 @@ where
 {
     unsafe fn clone_with_ptr(&self, ptr: NonNull<Self::Elem>) -> (Self, NonNull<Self::Elem>) {
         let mut u = self.clone();
-        let mut new_ptr = nonnull_from_vec_data(&mut u.0);
+        let mut new_ptr = u.as_nonnull_mut();
         if size_of::<A>() != 0 {
             let our_off =
-                (ptr.as_ptr() as isize - self.0.as_ptr() as isize) / mem::size_of::<A>() as isize;
+                (ptr.as_ptr() as isize - self.as_ptr() as isize) / mem::size_of::<A>() as isize;
             new_ptr = new_ptr.offset(our_off);
         }
         (u, new_ptr)
@@ -318,12 +317,12 @@ where
         ptr: NonNull<Self::Elem>,
     ) -> NonNull<Self::Elem> {
         let our_off = if size_of::<A>() != 0 {
-            (ptr.as_ptr() as isize - other.0.as_ptr() as isize) / mem::size_of::<A>() as isize
+            (ptr.as_ptr() as isize - other.as_ptr() as isize) / mem::size_of::<A>() as isize
         } else {
             0
         };
-        self.0.clone_from(&other.0);
-        nonnull_from_vec_data(&mut self.0).offset(our_off)
+        self.clone_from(&other);
+        self.as_nonnull_mut().offset(our_off)
     }
 }
 
@@ -413,16 +412,16 @@ unsafe impl<'a, A> DataShared for ViewRepr<&'a A> {}
 
 unsafe impl<A> DataOwned for OwnedRepr<A> {
     fn new(elements: Vec<A>) -> Self {
-        OwnedRepr(elements)
+        OwnedRepr::from(elements)
     }
     fn into_shared(self) -> OwnedRcRepr<A> {
-        OwnedArcRepr(Arc::new(self.0))
+        OwnedArcRepr(Arc::new(self))
     }
 }
 
 unsafe impl<A> DataOwned for OwnedArcRepr<A> {
     fn new(elements: Vec<A>) -> Self {
-        OwnedArcRepr(Arc::new(elements))
+        OwnedArcRepr(Arc::new(OwnedRepr::from(elements)))
     }
 
     fn into_shared(self) -> OwnedRcRepr<A> {
@@ -529,3 +528,38 @@ unsafe impl<'a, A> Data for CowRepr<'a, A> {
 }
 
 unsafe impl<'a, A> DataMut for CowRepr<'a, A> where A: Clone {}
+
+/// Array representation trait.
+///
+/// The RawDataSubst trait maps the element type of array storage, while
+/// keeping the same kind of storage.
+///
+/// For example, `RawDataSubst<B>` can map the type `OwnedRepr<A>` to `OwnedRepr<B>`.
+pub trait RawDataSubst<A>: RawData {
+    /// The resulting array storage of the same kind but substituted element type
+    type Output: RawData<Elem = A>;
+}
+
+impl<A, B> RawDataSubst<B> for OwnedRepr<A> {
+    type Output = OwnedRepr<B>;
+}
+
+impl<A, B> RawDataSubst<B> for OwnedArcRepr<A> {
+    type Output = OwnedArcRepr<B>;
+}
+
+impl<A, B> RawDataSubst<B> for RawViewRepr<*const A> {
+    type Output = RawViewRepr<*const B>;
+}
+
+impl<A, B> RawDataSubst<B> for RawViewRepr<*mut A> {
+    type Output = RawViewRepr<*mut B>;
+}
+
+impl<'a, A: 'a, B: 'a> RawDataSubst<B> for ViewRepr<&'a A> {
+    type Output = ViewRepr<&'a B>;
+}
+
+impl<'a, A: 'a, B: 'a> RawDataSubst<B> for ViewRepr<&'a mut A> {
+    type Output = ViewRepr<&'a mut B>;
+}
