@@ -1,13 +1,13 @@
-
-use {Dimension, Axis, Ix, Ixs};
+use crate::{Axis, Dimension, Ix, Ixs};
 
 /// Create a new Axes iterator
 pub fn axes_of<'a, D>(d: &'a D, strides: &'a D) -> Axes<'a, D>
-    where D: Dimension,
+where
+    D: Dimension,
 {
     Axes {
         dim: d,
-        strides: strides,
+        strides,
         start: 0,
         end: d.ndim(),
     }
@@ -33,7 +33,7 @@ pub fn axes_of<'a, D>(d: &'a D, strides: &'a D) -> Axes<'a, D>
 /// assert_eq!(largest_axis, Axis(1));
 /// ```
 #[derive(Debug)]
-pub struct Axes<'a, D: 'a> {
+pub struct Axes<'a, D> {
     dim: &'a D,
     strides: &'a D,
     start: usize,
@@ -46,22 +46,32 @@ pub struct AxisDescription(pub Axis, pub Ix, pub Ixs);
 
 copy_and_clone!(AxisDescription);
 
+// AxisDescription can't really be empty
+// https://github.com/rust-ndarray/ndarray/pull/642#discussion_r296051702
+#[allow(clippy::len_without_is_empty)]
 impl AxisDescription {
     /// Return axis
     #[inline(always)]
-    pub fn axis(self) -> Axis { self.0 }
+    pub fn axis(self) -> Axis {
+        self.0
+    }
     /// Return length
     #[inline(always)]
-    pub fn len(self) -> Ix { self.1 }
+    pub fn len(self) -> Ix {
+        self.1
+    }
     /// Return stride
     #[inline(always)]
-    pub fn stride(self) -> Ixs { self.2 }
+    pub fn stride(self) -> Ixs {
+        self.2
+    }
 }
 
 copy_and_clone!(['a, D] Axes<'a, D>);
 
 impl<'a, D> Iterator for Axes<'a, D>
-    where D: Dimension,
+where
+    D: Dimension,
 {
     /// Description of the axis, its length and its stride.
     type Item = AxisDescription;
@@ -69,12 +79,25 @@ impl<'a, D> Iterator for Axes<'a, D>
     fn next(&mut self) -> Option<Self::Item> {
         if self.start < self.end {
             let i = self.start.post_inc();
-            Some(AxisDescription(Axis(i), self.dim[i], self.strides[i] as Ixs))
+            Some(AxisDescription(
+                Axis(i),
+                self.dim[i],
+                self.strides[i] as Ixs,
+            ))
         } else {
             None
         }
     }
 
+    fn fold<B, F>(self, init: B, f: F) -> B
+    where
+        F: FnMut(B, AxisDescription) -> B,
+    {
+        (self.start..self.end)
+            .map(move |i| AxisDescription(Axis(i), self.dim[i], self.strides[i] as isize))
+            .fold(init, f)
+    }
+
     fn size_hint(&self) -> (usize, Option<usize>) {
         let len = self.end - self.start;
         (len, Some(len))
@@ -82,19 +105,24 @@ impl<'a, D> Iterator for Axes<'a, D>
 }
 
 impl<'a, D> DoubleEndedIterator for Axes<'a, D>
-    where D: Dimension,
+where
+    D: Dimension,
 {
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.start < self.end {
             let i = self.end.pre_dec();
-            Some(AxisDescription(Axis(i), self.dim[i], self.strides[i] as Ixs))
+            Some(AxisDescription(
+                Axis(i),
+                self.dim[i],
+                self.strides[i] as Ixs,
+            ))
         } else {
             None
         }
     }
 }
 
-trait IncOps : Copy {
+trait IncOps: Copy {
     fn post_inc(&mut self) -> Self;
     fn post_dec(&mut self) -> Self;
     fn pre_dec(&mut self) -> Self;
@@ -119,4 +147,3 @@ impl IncOps for usize {
         *self
     }
 }
-
