@@ -10,11 +10,10 @@ use std::fmt::Debug;
 use std::ops::{Add, AddAssign, Mul, MulAssign, Sub, SubAssign};
 use std::ops::{Index, IndexMut};
 
-use itertools::{enumerate, izip, zip};
-
 use super::axes_of;
 use super::conversion::Convert;
 use super::{stride_offset, stride_offset_checked};
+use crate::itertools::{enumerate, zip};
 use crate::Axis;
 use crate::IntoDimension;
 use crate::RemoveAxis;
@@ -76,7 +75,7 @@ pub trait Dimension:
     /// Next smaller dimension (if applicable)
     type Smaller: Dimension;
     /// Next larger dimension
-    type Larger: Dimension;
+    type Larger: Dimension + RemoveAxis;
 
     /// Returns the number of dimensions (number of axes).
     fn ndim(&self) -> usize;
@@ -230,6 +229,25 @@ pub trait Dimension:
         !end_iteration
     }
 
+    /// Returns `true` iff `strides1` and `strides2` are equivalent for the
+    /// shape `self`.
+    ///
+    /// The strides are equivalent if, for each axis with length > 1, the
+    /// strides are equal.
+    ///
+    /// Note: Returns `false` if any of the ndims don't match.
+    #[doc(hidden)]
+    fn strides_equivalent<D>(&self, strides1: &Self, strides2: &D) -> bool
+    where
+        D: Dimension,
+    {
+        let shape_ndim = self.ndim();
+        shape_ndim == strides1.ndim()
+            && shape_ndim == strides2.ndim()
+            && izip!(self.slice(), strides1.slice(), strides2.slice())
+                .all(|(&d, &s1, &s2)| d <= 1 || s1 as isize == s2 as isize)
+    }
+
     #[doc(hidden)]
     /// Return stride offset for index.
     fn stride_offset(index: &Self, strides: &Self) -> isize {
