@@ -8,7 +8,6 @@
 
 use crate::error::{from_kind, ErrorKind, ShapeError};
 use crate::{Ix, Ixs, Slice, SliceOrIndex};
-use itertools::izip;
 use num_integer::div_floor;
 
 pub use self::axes::{axes_of, Axes, AxisDescription};
@@ -545,7 +544,6 @@ fn slice_min_max(axis_len: usize, slice: Slice) -> Option<(usize, usize)> {
 }
 
 /// Returns `true` iff the slices intersect.
-#[allow(dead_code)]
 pub fn slices_intersect<D: Dimension>(
     dim: &D,
     indices1: &D::SliceArg,
@@ -632,7 +630,6 @@ where
     }
 }
 
-// NOTE: These tests are not compiled & tested
 #[cfg(test)]
 mod test {
     use super::{
