@@ -13,6 +13,7 @@
 #![allow(clippy::match_wild_err_arm)]
 
 use num_traits::{Float, One, Zero};
+use std::mem::MaybeUninit;
 
 use crate::dimension;
 use crate::error::{self, ShapeError};
@@ -143,7 +144,7 @@ where
     /// # Some(())
     /// # }
     /// #
-    /// # fn main() { example().unwrap() }
+    /// # example().unwrap();
     /// ```
     pub fn geomspace(start: A, end: A, n: usize) -> Option<Self>
     where
@@ -305,10 +306,28 @@ where
     where
         A: Default,
         Sh: ShapeBuilder<Dim = D>,
+    {
+        Self::from_shape_simple_fn(shape, A::default)
+    }
+
+    /// Create an array with values created by the function `f`.
+    ///
+    /// `f` is called with no argument, and it should return the element to
+    /// create. If the precise index of the element to create is needed,
+    /// use [`from_shape_fn`](ArrayBase::from_shape_fn) instead.
+    ///
+    /// This constructor can be useful if the element order is not important,
+    /// for example if they are identical or random.
+    ///
+    /// **Panics** if the product of non-zero axis lengths overflows `isize`.
+    pub fn from_shape_simple_fn<Sh, F>(shape: Sh, mut f: F) -> Self
+    where
+        Sh: ShapeBuilder<Dim = D>,
+        F: FnMut() -> A,
     {
         let shape = shape.into_shape();
-        let size = size_of_shape_checked_unwrap!(&shape.dim);
-        let v = to_vec((0..size).map(|_| A::default()));
+        let len = size_of_shape_checked_unwrap!(&shape.dim);
+        let v = to_vec_mapped(0..len, move |_| f());
         unsafe { Self::from_shape_vec_unchecked(shape, v) }
     }
 
@@ -318,6 +337,20 @@ where
     /// visited in arbitrary order.
     ///
     /// **Panics** if the product of non-zero axis lengths overflows `isize`.
+    ///
+    /// ```
+    /// use ndarray::{Array, arr2};
+    ///
+    /// // Create a table of i × j (with i and j from 1 to 3)
+    /// let ij_table = Array::from_shape_fn((3, 3), |(i, j)| (1 + i) * (1 + j));
+    ///
+    /// assert_eq!(
+    ///     ij_table,
+    ///     arr2(&[[1, 2, 3],
+    ///            [2, 4, 6],
+    ///            [3, 6, 9]])
+    /// );
+    /// ```
     pub fn from_shape_fn<Sh, F>(shape: Sh, f: F) -> Self
     where
         Sh: ShapeBuilder<Dim = D>,
@@ -393,6 +426,8 @@ where
     /// Creates an array from a vector and interpret it according to the
     /// provided shape and strides. (No cloning of elements needed.)
     ///
+    /// # Safety
+    ///
     /// The caller must ensure that the following conditions are met:
     ///
     /// 1. The ndim of `dim` and `strides` must be the same.
@@ -432,49 +467,27 @@ where
 
     /// Create an array with uninitalized elements, shape `shape`.
     ///
+    /// Prefer to use [`maybe_uninit()`](ArrayBase::maybe_uninit) if possible, because it is
+    /// easier to use correctly.
+    ///
     /// **Panics** if the number of elements in `shape` would overflow isize.
     ///
     /// ### Safety
     ///
-    /// Accessing uninitalized values is undefined behaviour. You must
-    /// overwrite *all* the elements in the array after it is created; for
-    /// example using the methods `.fill()` or `.assign()`.
+    /// Accessing uninitalized values is undefined behaviour. You must overwrite *all* the elements
+    /// in the array after it is created; for example using
+    /// [`raw_view_mut`](ArrayBase::raw_view_mut) or other low-level element access.
     ///
     /// The contents of the array is indeterminate before initialization and it
     /// is an error to perform operations that use the previous values. For
     /// example it would not be legal to use `a += 1.;` on such an array.
     ///
     /// This constructor is limited to elements where `A: Copy` (no destructors)
-    /// to avoid users shooting themselves too hard in the foot; it is not
-    /// a problem to drop an array created with this method even before elements
-    /// are initialized. (Note that constructors `from_shape_vec` and
-    /// `from_shape_vec_unchecked` allow the user yet more control).
-    ///
-    /// ### Examples
-    ///
-    /// ```
-    /// extern crate ndarray;
-    ///
-    /// use ndarray::{s, Array2};
-    ///
-    /// // Example Task: Let's create a column shifted copy of a in b
-    ///
-    /// fn shift_by_two(a: &Array2<f32>) -> Array2<f32> {
-    ///     let mut b = unsafe { Array2::uninitialized(a.dim()) };
-    ///
-    ///     // two first columns in b are two last in a
-    ///     // rest of columns in b are the initial columns in a
-    ///     b.slice_mut(s![.., ..2]).assign(&a.slice(s![.., -2..]));
-    ///     b.slice_mut(s![.., 2..]).assign(&a.slice(s![.., ..-2]));
-    ///
-    ///     // `b` is safe to use with all operations at this point
-    ///     b
-    /// }
-    ///
-    /// # fn main() {
-    /// #   shift_by_two(&Array2::zeros((8, 8)));
-    /// # }
-    /// ```
+    /// to avoid users shooting themselves too hard in the foot.
+    /// 
+    /// (Also note that the constructors `from_shape_vec` and
+    /// `from_shape_vec_unchecked` allow the user yet more control, in the sense
+    /// that Arrays can be created from arbitrary vectors.)
     pub unsafe fn uninitialized<Sh>(shape: Sh) -> Self
     where
         A: Copy,
@@ -487,3 +500,83 @@ where
         Self::from_shape_vec_unchecked(shape, v)
     }
 }
+
+impl<S, A, D> ArrayBase<S, D>
+where
+    S: DataOwned<Elem = MaybeUninit<A>>,
+    D: Dimension,
+{
+    /// Create an array with uninitalized elements, shape `shape`.
+    ///
+    /// The uninitialized elements of type `A` are represented by the type `MaybeUninit<A>`,
+    /// an easier way to handle uninit values correctly.
+    ///
+    /// Only *when* the array is completely initialized with valid elements, can it be
+    /// converted to an array of `A` elements using [`.assume_init()`].
+    ///
+    /// **Panics** if the number of elements in `shape` would overflow isize.
+    ///
+    /// ### Safety
+    ///
+    /// The whole of the array must be initialized before it is converted
+    /// using [`.assume_init()`] or otherwise traversed.
+    ///
+    /// ### Examples
+    ///
+    /// It is possible to assign individual values through `*elt = MaybeUninit::new(value)`
+    /// and so on.
+    ///
+    /// [`.assume_init()`]: ArrayBase::assume_init
+    ///
+    /// ```
+    /// use ndarray::{s, Array2};
+    /// use ndarray::Zip;
+    /// use ndarray::Axis;
+    ///
+    /// // Example Task: Let's create a column shifted copy of the input
+    ///
+    /// fn shift_by_two(a: &Array2<f32>) -> Array2<f32> {
+    ///     // create an uninitialized array
+    ///     let mut b = Array2::maybe_uninit(a.dim());
+    ///
+    ///     // two first columns in b are two last in a
+    ///     // rest of columns in b are the initial columns in a
+    ///
+    ///     assign_to(a.slice(s![.., -2..]), b.slice_mut(s![.., ..2]));
+    ///     assign_to(a.slice(s![.., 2..]), b.slice_mut(s![.., ..-2]));
+    ///
+    ///     // Now we can promise that `b` is safe to use with all operations
+    ///     unsafe {
+    ///         b.assume_init()
+    ///     }
+    /// }
+    ///
+    /// use ndarray::{IntoNdProducer, AssignElem};
+    ///
+    /// // This function clones elements from the first input to the second;
+    /// // the two producers must have the same shape
+    /// fn assign_to<'a, P1, P2, A>(from: P1, to: P2)
+    ///     where P1: IntoNdProducer<Item = &'a A>,
+    ///           P2: IntoNdProducer<Dim = P1::Dim>,
+    ///           P2::Item: AssignElem<A>,
+    ///           A: Clone + 'a
+    /// {
+    ///     Zip::from(from)
+    ///         .apply_assign_into(to, A::clone);
+    /// }
+    ///
+    /// # shift_by_two(&Array2::zeros((8, 8)));
+    /// ```
+    pub fn maybe_uninit<Sh>(shape: Sh) -> Self
+    where
+        Sh: ShapeBuilder<Dim = D>,
+    {
+        unsafe {
+            let shape = shape.into_shape();
+            let size = size_of_shape_checked_unwrap!(&shape.dim);
+            let mut v = Vec::with_capacity(size);
+            v.set_len(size);
+            Self::from_shape_vec_unchecked(shape, v)
+        }
+    }
+}
