@@ -80,6 +80,32 @@ where
         };
         (l, Some(l))
     }
+
+    fn fold<B, F>(self, init: B, mut f: F) -> B
+    where
+        F: FnMut(B, D::Pattern) -> B,
+    {
+        let IndicesIter { mut index, dim } = self;
+        let ndim = dim.ndim();
+        if ndim == 0 {
+            return match index {
+                Some(ix) => f(init, ix.into_pattern()),
+                None => init,
+            };
+        }
+        let inner_axis = ndim - 1;
+        let inner_len = dim[inner_axis];
+        let mut acc = init;
+        while let Some(mut ix) = index {
+            // unroll innermost axis
+            for i in ix[inner_axis]..inner_len {
+                ix[inner_axis] = i;
+                acc = f(acc, ix.clone().into_pattern());
+            }
+            index = dim.next_for(ix);
+        }
+        acc
+    }
 }
 
 impl<D> ExactSizeIterator for IndicesIter<D> where D: Dimension {}
@@ -247,18 +273,14 @@ where
         if !self.has_remaining {
             return (0, Some(0));
         }
-        let l = match self.index {
-            ref ix => {
-                let gone = self
-                    .dim
-                    .fortran_strides()
-                    .slice()
-                    .iter()
-                    .zip(ix.slice().iter())
-                    .fold(0, |s, (&a, &b)| s + a as usize * b as usize);
-                self.dim.size() - gone
-            }
-        };
+        let gone = self
+            .dim
+            .fortran_strides()
+            .slice()
+            .iter()
+            .zip(self.index.slice().iter())
+            .fold(0, |s, (&a, &b)| s + a as usize * b as usize);
+        let l = self.dim.size() - gone;
         (l, Some(l))
     }
 }
@@ -283,6 +305,34 @@ mod tests {
         assert_eq!(len, 0);
     }
 
+    #[test]
+    fn test_indices_iter_c_fold() {
+        macro_rules! run_test {
+            ($dim:expr) => {
+                for num_consume in 0..3 {
+                    let mut it = indices($dim).into_iter();
+                    for _ in 0..num_consume {
+                        it.next();
+                    }
+                    let clone = it.clone();
+                    let len = it.len();
+                    let acc = clone.fold(0, |acc, ix| {
+                        assert_eq!(ix, it.next().unwrap());
+                        acc + 1
+                    });
+                    assert_eq!(acc, len);
+                    assert!(it.next().is_none());
+                }
+            };
+        }
+        run_test!(());
+        run_test!((2,));
+        run_test!((2, 3));
+        run_test!((2, 0, 3));
+        run_test!((2, 3, 4));
+        run_test!((2, 3, 4, 2));
+    }
+
     #[test]
     fn test_indices_iter_f_size_hint() {
         let dim = (3, 4);
