@@ -1,17 +1,23 @@
-
-#[macro_use(array, s)]
-extern crate ndarray;
+#![allow(
+    clippy::many_single_char_names,
+    clippy::deref_addrof,
+    clippy::unreadable_literal,
+    clippy::many_single_char_names,
+    clippy::float_cmp
+)]
 
 use ndarray::prelude::*;
 use ndarray::NdProducer;
 
 #[test]
 fn chunks() {
-    let a = <Array1<f32>>::linspace(1., 100., 10 * 10).into_shape((10, 10)).unwrap();
+    let a = <Array1<f32>>::linspace(1., 100., 10 * 10)
+        .into_shape((10, 10))
+        .unwrap();
 
     let (m, n) = a.dim();
-    for i in 1..m + 1 {
-        for j in 1..n + 1 {
+    for i in 1..=m {
+        for j in 1..=n {
             let c = a.exact_chunks((i, j));
 
             let ly = n / j;
@@ -20,7 +26,11 @@ fn chunks() {
                 let cindex = (index / ly, index % ly);
                 let cx = (cindex.0 * i) as isize;
                 let cy = (cindex.1 * j) as isize;
-                assert_eq!(elt, a.slice(s![cx.., cy..]).slice(s![..i as isize, ..j as isize]));
+                assert_eq!(
+                    elt,
+                    a.slice(s![cx.., cy..])
+                        .slice(s![..i as isize, ..j as isize])
+                );
             }
             let c = a.exact_chunks((i, j));
             assert_eq!(c.into_iter().count(), (m / i) * (n / j));
@@ -71,13 +81,14 @@ fn chunks_mut() {
     }
     println!("{:?}", a);
     let ans = array![
-       [0, 0, 0, 1, 1, 1, 0, 0],
-       [0, 0, 0, 1, 1, 1, 0, 0],
-       [2, 2, 2, 3, 3, 3, 0, 0],
-       [2, 2, 2, 3, 3, 3, 0, 0],
-       [4, 4, 4, 5, 5, 5, 0, 0],
-       [4, 4, 4, 5, 5, 5, 0, 0],
-       [0, 0, 0, 0, 0, 0, 0, 0]];
+        [0, 0, 0, 1, 1, 1, 0, 0],
+        [0, 0, 0, 1, 1, 1, 0, 0],
+        [2, 2, 2, 3, 3, 3, 0, 0],
+        [2, 2, 2, 3, 3, 3, 0, 0],
+        [4, 4, 4, 5, 5, 5, 0, 0],
+        [4, 4, 4, 5, 5, 5, 0, 0],
+        [0, 0, 0, 0, 0, 0, 0, 0]
+    ];
     assert_eq!(a, ans);
 }
 
