@@ -1,20 +1,22 @@
-#[macro_use] extern crate defmac;
-#[macro_use(s)] extern crate ndarray;
-extern crate num_traits;
+#![allow(
+    clippy::many_single_char_names,
+    clippy::deref_addrof,
+    clippy::unreadable_literal,
+    clippy::many_single_char_names
+)]
 
+use ndarray::linalg::general_mat_mul;
 use ndarray::prelude::*;
 use ndarray::{rcarr1, rcarr2};
-use ndarray::{LinalgScalar, Data};
-use ndarray::linalg::general_mat_mul;
-use ndarray::linalg::general_mat_vec_mul;
-use ndarray::{Ix, Ixs, SliceInfo, SliceOrIndex};
+use ndarray::{Data, LinalgScalar};
+use ndarray::{Ix, Ixs};
+use std::iter::FromIterator;
 
-use std::fmt;
+use approx::assert_abs_diff_eq;
+use defmac::defmac;
 use std::ops::Neg;
-use num_traits::Float;
 
-fn test_oper(op: &str, a: &[f32], b: &[f32], c: &[f32])
-{
+fn test_oper(op: &str, a: &[f32], b: &[f32], c: &[f32]) {
     let aa = rcarr1(a);
     let bb = rcarr1(b);
     let cc = rcarr1(c);
@@ -31,10 +33,10 @@ fn test_oper(op: &str, a: &[f32], b: &[f32], c: &[f32])
     test_oper_arr::<f32, _>(op, aa.clone(), bb.clone(), cc.clone());
 }
 
-fn test_oper_arr<A, D>(op: &str, mut aa: RcArray<A, D>, bb: RcArray<A, D>, cc: RcArray<A, D>)
+fn test_oper_arr<A, D>(op: &str, mut aa: ArcArray<A, D>, bb: ArcArray<A, D>, cc: ArcArray<A, D>)
 where
     A: NdFloat,
-    for<'a> &'a A: Neg<Output=A>,
+    for<'a> &'a A: Neg<Output = A>,
     D: Dimension,
 {
     match op {
@@ -42,49 +44,77 @@ where
             assert_eq!(&aa + &bb, cc);
             aa += &bb;
             assert_eq!(aa, cc);
-        },
+        }
         "-" => {
             assert_eq!(&aa - &bb, cc);
             aa -= &bb;
             assert_eq!(aa, cc);
-        },
+        }
         "*" => {
             assert_eq!(&aa * &bb, cc);
             aa *= &bb;
             assert_eq!(aa, cc);
-        },
+        }
         "/" => {
             assert_eq!(&aa / &bb, cc);
             aa /= &bb;
             assert_eq!(aa, cc);
-        },
+        }
         "%" => {
             assert_eq!(&aa % &bb, cc);
             aa %= &bb;
             assert_eq!(aa, cc);
-        },
+        }
         "neg" => {
             assert_eq!(-&aa, cc);
             assert_eq!(-aa.clone(), cc);
-        },
-        _ => panic!()
+        }
+        _ => panic!(),
     }
 }
 
 #[test]
-fn operations()
-{
-    test_oper("+", &[1.0,2.0,3.0,4.0], &[0.0, 1.0, 2.0, 3.0], &[1.0,3.0,5.0,7.0]);
-    test_oper("-", &[1.0,2.0,3.0,4.0], &[0.0, 1.0, 2.0, 3.0], &[1.0,1.0,1.0,1.0]);
-    test_oper("*", &[1.0,2.0,3.0,4.0], &[0.0, 1.0, 2.0, 3.0], &[0.0,2.0,6.0,12.0]);
-    test_oper("/", &[1.0,2.0,3.0,4.0], &[1.0, 1.0, 2.0, 3.0], &[1.0,2.0,3.0/2.0,4.0/3.0]);
-    test_oper("%", &[1.0,2.0,3.0,4.0], &[1.0, 1.0, 2.0, 3.0], &[0.0,0.0,1.0,1.0]);
-    test_oper("neg", &[1.0,2.0,3.0,4.0], &[1.0, 1.0, 2.0, 3.0], &[-1.0,-2.0,-3.0,-4.0]);
+fn operations() {
+    test_oper(
+        "+",
+        &[1.0, 2.0, 3.0, 4.0],
+        &[0.0, 1.0, 2.0, 3.0],
+        &[1.0, 3.0, 5.0, 7.0],
+    );
+    test_oper(
+        "-",
+        &[1.0, 2.0, 3.0, 4.0],
+        &[0.0, 1.0, 2.0, 3.0],
+        &[1.0, 1.0, 1.0, 1.0],
+    );
+    test_oper(
+        "*",
+        &[1.0, 2.0, 3.0, 4.0],
+        &[0.0, 1.0, 2.0, 3.0],
+        &[0.0, 2.0, 6.0, 12.0],
+    );
+    test_oper(
+        "/",
+        &[1.0, 2.0, 3.0, 4.0],
+        &[1.0, 1.0, 2.0, 3.0],
+        &[1.0, 2.0, 3.0 / 2.0, 4.0 / 3.0],
+    );
+    test_oper(
+        "%",
+        &[1.0, 2.0, 3.0, 4.0],
+        &[1.0, 1.0, 2.0, 3.0],
+        &[0.0, 0.0, 1.0, 1.0],
+    );
+    test_oper(
+        "neg",
+        &[1.0, 2.0, 3.0, 4.0],
+        &[1.0, 1.0, 2.0, 3.0],
+        &[-1.0, -2.0, -3.0, -4.0],
+    );
 }
 
 #[test]
-fn scalar_operations()
-{
+fn scalar_operations() {
     let a = arr0::<f32>(1.);
     let b = rcarr1::<f32>(&[1., 1.]);
     let c = rcarr2(&[[1., 1.], [1., 1.]]);
@@ -109,7 +139,7 @@ fn scalar_operations()
 
     {
         let mut x = c.clone();
-        let mut y = RcArray::zeros((2, 2));
+        let mut y = ArcArray::zeros((2, 2));
         x += 1.;
         y.fill(2.);
         assert_eq!(x, c + arr0(1.));
@@ -117,41 +147,17 @@ fn scalar_operations()
     }
 }
 
-fn assert_approx_eq<F: fmt::Debug + Float>(f: F, g: F, tol: F) -> bool {
-    assert!((f - g).abs() <= tol, "{:?} approx== {:?} (tol={:?})",
-            f, g, tol);
-    true
-}
-
-fn assert_close<D>(a: ArrayView<f64, D>, b: ArrayView<f64, D>)
-    where D: Dimension,
-{
-    let diff = (&a - &b).mapv_into(f64::abs);
-
-    let rtol = 1e-7;
-    let atol = 1e-12;
-    let crtol = b.mapv(|x| x.abs() * rtol);
-    let tol = crtol + atol;
-    let tol_m_diff = &diff - &tol;
-    let maxdiff = tol_m_diff.fold(0./0., |x, y| f64::max(x, *y));
-    println!("diff offset from tolerance level= {:.2e}", maxdiff);
-    if maxdiff > 0. {
-        println!("{:.4?}", a);
-        println!("{:.4?}", b);
-        panic!("results differ");
-    }
-}
-
-fn reference_dot<'a,A, V1, V2>(a: V1, b: V2) -> A
-    where A: NdFloat,
-          V1: AsArray<'a, A>,
-          V2: AsArray<'a, A>,
+fn reference_dot<'a, A, V1, V2>(a: V1, b: V2) -> A
+where
+    A: NdFloat,
+    V1: AsArray<'a, A>,
+    V2: AsArray<'a, A>,
 {
     let a = a.into();
     let b = b.into();
-    a.iter().zip(b.iter()).fold(A::zero(), |acc, (&x, &y)| {
-        acc + x * y
-    })
+    a.iter()
+        .zip(b.iter())
+        .fold(A::zero(), |acc, (&x, &y)| acc + x * y)
 }
 
 #[test]
@@ -159,32 +165,31 @@ fn dot_product() {
     let a = Array::range(0., 69., 1.);
     let b = &a * 2. - 7.;
     let dot = 197846.;
-    assert_approx_eq(a.dot(&b), reference_dot(&a, &b), 1e-5);
+    assert_abs_diff_eq!(a.dot(&b), reference_dot(&a, &b), epsilon = 1e-5);
 
     // test different alignments
     let max = 8 as Ixs;
     for i in 1..max {
         let a1 = a.slice(s![i..]);
         let b1 = b.slice(s![i..]);
-        assert_approx_eq(a1.dot(&b1), reference_dot(&a1, &b1), 1e-5);
+        assert_abs_diff_eq!(a1.dot(&b1), reference_dot(&a1, &b1), epsilon = 1e-5);
         let a2 = a.slice(s![..-i]);
         let b2 = b.slice(s![i..]);
-        assert_approx_eq(a2.dot(&b2), reference_dot(&a2, &b2), 1e-5);
+        assert_abs_diff_eq!(a2.dot(&b2), reference_dot(&a2, &b2), epsilon = 1e-5);
     }
 
-
     let a = a.map(|f| *f as f32);
     let b = b.map(|f| *f as f32);
-    assert_approx_eq(a.dot(&b), dot as f32, 1e-5);
+    assert_abs_diff_eq!(a.dot(&b), dot as f32, epsilon = 1e-5);
 
     let max = 8 as Ixs;
     for i in 1..max {
         let a1 = a.slice(s![i..]);
         let b1 = b.slice(s![i..]);
-        assert_approx_eq(a1.dot(&b1), reference_dot(&a1, &b1), 1e-5);
+        assert_abs_diff_eq!(a1.dot(&b1), reference_dot(&a1, &b1), epsilon = 1e-5);
         let a2 = a.slice(s![..-i]);
         let b2 = b.slice(s![i..]);
-        assert_approx_eq(a2.dot(&b2), reference_dot(&a2, &b2), 1e-5);
+        assert_abs_diff_eq!(a2.dot(&b2), reference_dot(&a2, &b2), epsilon = 1e-5);
     }
 
     let a = a.map(|f| *f as i32);
@@ -199,17 +204,17 @@ fn dot_product_0() {
     let x = 1.5;
     let b = aview0(&x);
     let b = b.broadcast(a.dim()).unwrap();
-    assert_approx_eq(a.dot(&b), reference_dot(&a, &b), 1e-5);
+    assert_abs_diff_eq!(a.dot(&b), reference_dot(&a, &b), epsilon = 1e-5);
 
     // test different alignments
     let max = 8 as Ixs;
     for i in 1..max {
         let a1 = a.slice(s![i..]);
         let b1 = b.slice(s![i..]);
-        assert_approx_eq(a1.dot(&b1), reference_dot(&a1, &b1), 1e-5);
+        assert_abs_diff_eq!(a1.dot(&b1), reference_dot(&a1, &b1), epsilon = 1e-5);
         let a2 = a.slice(s![..-i]);
         let b2 = b.slice(s![i..]);
-        assert_approx_eq(a2.dot(&b2), reference_dot(&a2, &b2), 1e-5);
+        assert_abs_diff_eq!(a2.dot(&b2), reference_dot(&a2, &b2), epsilon = 1e-5);
     }
 }
 
@@ -222,20 +227,20 @@ fn dot_product_neg_stride() {
         // both negative
         let a = a.slice(s![..;stride]);
         let b = b.slice(s![..;stride]);
-        assert_approx_eq(a.dot(&b), reference_dot(&a, &b), 1e-5);
+        assert_abs_diff_eq!(a.dot(&b), reference_dot(&a, &b), epsilon = 1e-5);
     }
     for stride in -10..0 {
         // mixed
         let a = a.slice(s![..;-stride]);
         let b = b.slice(s![..;stride]);
-        assert_approx_eq(a.dot(&b), reference_dot(&a, &b), 1e-5);
+        assert_abs_diff_eq!(a.dot(&b), reference_dot(&a, &b), epsilon = 1e-5);
     }
 }
 
 #[test]
 fn fold_and_sum() {
     let a = Array::linspace(0., 127., 128).into_shape((8, 16)).unwrap();
-    assert_approx_eq(a.fold(0., |acc, &x| acc +x), a.sum(), 1e-5);
+    assert_abs_diff_eq!(a.fold(0., |acc, &x| acc + x), a.sum(), epsilon = 1e-5);
 
     // test different strides
     let max = 8 as Ixs;
@@ -246,8 +251,8 @@ fn fold_and_sum() {
             for elt in a1.iter() {
                 sum += *elt;
             }
-            assert_approx_eq(a1.fold(0., |acc, &x| acc +x), sum, 1e-5);
-            assert_approx_eq(sum, a1.sum(), 1e-5);
+            assert_abs_diff_eq!(a1.fold(0., |acc, &x| acc + x), sum, epsilon = 1e-5);
+            assert_abs_diff_eq!(sum, a1.sum(), epsilon = 1e-5);
         }
     }
 
@@ -266,7 +271,7 @@ fn fold_and_sum() {
             for elt in iter1 {
                 sum += *elt;
             }
-            assert_approx_eq(iter2.fold(0., |acc, &x| acc +x), sum, 1e-5);
+            assert_abs_diff_eq!(iter2.fold(0., |acc, &x| acc + x), sum, epsilon = 1e-5);
         }
     }
 }
@@ -274,7 +279,7 @@ fn fold_and_sum() {
 #[test]
 fn product() {
     let a = Array::linspace(0.5, 2., 128).into_shape((8, 16)).unwrap();
-    assert_approx_eq(a.fold(1., |acc, &x| acc * x), a.product(), 1e-5);
+    assert_abs_diff_eq!(a.fold(1., |acc, &x| acc * x), a.product(), epsilon = 1e-5);
 
     // test different strides
     let max = 8 as Ixs;
@@ -285,34 +290,41 @@ fn product() {
             for elt in a1.iter() {
                 prod *= *elt;
             }
-            assert_approx_eq(a1.fold(1., |acc, &x| acc * x), prod, 1e-5);
-            assert_approx_eq(prod, a1.product(), 1e-5);
+            assert_abs_diff_eq!(a1.fold(1., |acc, &x| acc * x), prod, epsilon = 1e-5);
+            assert_abs_diff_eq!(prod, a1.product(), epsilon = 1e-5);
         }
     }
 }
 
 fn range_mat(m: Ix, n: Ix) -> Array2<f32> {
-    Array::linspace(0., (m * n) as f32 - 1., m * n).into_shape((m, n)).unwrap()
+    Array::linspace(0., (m * n) as f32 - 1., m * n)
+        .into_shape((m, n))
+        .unwrap()
 }
 
 fn range_mat64(m: Ix, n: Ix) -> Array2<f64> {
-    Array::linspace(0., (m * n) as f64 - 1., m * n).into_shape((m, n)).unwrap()
+    Array::linspace(0., (m * n) as f64 - 1., m * n)
+        .into_shape((m, n))
+        .unwrap()
 }
 
+#[cfg(feature = "approx")]
 fn range1_mat64(m: Ix) -> Array1<f64> {
     Array::linspace(0., m as f64 - 1., m)
 }
 
 fn range_i32(m: Ix, n: Ix) -> Array2<i32> {
-    Array::from_iter(0..(m * n) as i32).into_shape((m, n)).unwrap()
+    Array::from_iter(0..(m * n) as i32)
+        .into_shape((m, n))
+        .unwrap()
 }
 
 // simple, slow, correct (hopefully) mat mul
-fn reference_mat_mul<A, S, S2>(lhs: &ArrayBase<S, Ix2>, rhs: &ArrayBase<S2, Ix2>)
-    -> Array2<A>
-    where A: LinalgScalar,
-          S: Data<Elem=A>,
-          S2: Data<Elem=A>,
+fn reference_mat_mul<A, S, S2>(lhs: &ArrayBase<S, Ix2>, rhs: &ArrayBase<S2, Ix2>) -> Array2<A>
+where
+    A: LinalgScalar,
+    S: Data<Elem = A>,
+    S2: Data<Elem = A>,
 {
     let ((m, k), (k2, n)) = (lhs.dim(), rhs.dim());
     assert!(m.checked_mul(n).is_some());
@@ -326,8 +338,9 @@ fn reference_mat_mul<A, S, S2>(lhs: &ArrayBase<S, Ix2>, rhs: &ArrayBase<S2, Ix2>
     let mut j = 0;
     for rr in &mut res_elems {
         unsafe {
-            *rr = (0..k).fold(A::zero(),
-                move |s, x| s + *lhs.uget((i, x)) * *rhs.uget((x, j)));
+            *rr = (0..k).fold(A::zero(), move |s, x| {
+                s + *lhs.uget((i, x)) * *rhs.uget((x, j))
+            });
         }
         j += 1;
         if j == n {
@@ -335,33 +348,7 @@ fn reference_mat_mul<A, S, S2>(lhs: &ArrayBase<S, Ix2>, rhs: &ArrayBase<S2, Ix2>
             i += 1;
         }
     }
-    unsafe {
-        ArrayBase::from_shape_vec_unchecked((m, n), res_elems)
-    }
-}
-
-// simple, slow, correct (hopefully) mat mul
-fn reference_mat_vec_mul<A, S, S2>(lhs: &ArrayBase<S, Ix2>, rhs: &ArrayBase<S2, Ix1>)
-    -> Array1<A>
-    where A: LinalgScalar,
-          S: Data<Elem=A>,
-          S2: Data<Elem=A>,
-{
-    let ((m, _), k) = (lhs.dim(), rhs.dim());
-    reference_mat_mul(lhs, &rhs.to_owned().into_shape((k, 1)).unwrap())
-        .into_shape(m).unwrap()
-}
-
-// simple, slow, correct (hopefully) mat mul
-fn reference_vec_mat_mul<A, S, S2>(lhs: &ArrayBase<S, Ix1>, rhs: &ArrayBase<S2, Ix2>)
-    -> Array1<A>
-    where A: LinalgScalar,
-          S: Data<Elem=A>,
-          S2: Data<Elem=A>,
-{
-    let (m, (_, n)) = (lhs.dim(), rhs.dim());
-    reference_mat_mul(&lhs.to_owned().into_shape((1, m)).unwrap(), rhs)
-        .into_shape(n).unwrap()
+    unsafe { ArrayBase::from_shape_vec_unchecked((m, n), res_elems) }
 }
 
 #[test]
@@ -471,7 +458,7 @@ fn mat_mul_broadcast() {
     let (m, n, k) = (16, 16, 16);
     let a = range_mat(m, n);
     let x1 = 1.;
-    let x = Array::from_vec(vec![x1]);
+    let x = Array::from(vec![x1]);
     let b0 = x.broadcast((n, k)).unwrap();
     let b1 = Array::from_elem(n, x1);
     let b1 = b1.broadcast((n, k)).unwrap();
@@ -534,22 +521,23 @@ fn scaled_add() {
 
     let d = alpha * &b + &a;
     assert_eq!(c, d);
-
 }
 
+#[cfg(feature = "approx")]
 #[test]
 fn scaled_add_2() {
     let beta = -2.3;
-    let sizes = vec![(4, 4, 1, 4),
-                     (8, 8, 1, 8),
-                     (17, 15, 17, 15),
-                     (4, 17, 4, 17),
-                     (17, 3, 1, 3),
-                     (19, 18, 19, 18),
-                     (16, 17, 16, 17),
-                     (15, 16, 15, 16),
-                     (67, 63, 1, 63),
-        ];
+    let sizes = vec![
+        (4, 4, 1, 4),
+        (8, 8, 1, 8),
+        (17, 15, 17, 15),
+        (4, 17, 4, 17),
+        (17, 3, 1, 3),
+        (19, 18, 19, 18),
+        (16, 17, 16, 17),
+        (15, 16, 15, 16),
+        (67, 63, 1, 63),
+    ];
     // test different strides
     for &s1 in &[1, 2, -1, -2] {
         for &s2 in &[1, 2, -1, -2] {
@@ -566,36 +554,37 @@ fn scaled_add_2() {
                     answerv += &(beta * &c);
                     av.scaled_add(beta, &c);
                 }
-                assert_close(a.view(), answer.view());
+                approx::assert_relative_eq!(a, answer, epsilon = 1e-12, max_relative = 1e-7);
             }
         }
     }
 }
 
+#[cfg(feature = "approx")]
 #[test]
 fn scaled_add_3() {
+    use approx::assert_relative_eq;
+    use ndarray::{SliceInfo, SliceOrIndex};
+
     let beta = -2.3;
-    let sizes = vec![(4, 4, 1, 4),
-                     (8, 8, 1, 8),
-                     (17, 15, 17, 15),
-                     (4, 17, 4, 17),
-                     (17, 3, 1, 3),
-                     (19, 18, 19, 18),
-                     (16, 17, 16, 17),
-                     (15, 16, 15, 16),
-                     (67, 63, 1, 63),
-        ];
+    let sizes = vec![
+        (4, 4, 1, 4),
+        (8, 8, 1, 8),
+        (17, 15, 17, 15),
+        (4, 17, 4, 17),
+        (17, 3, 1, 3),
+        (19, 18, 19, 18),
+        (16, 17, 16, 17),
+        (15, 16, 15, 16),
+        (67, 63, 1, 63),
+    ];
     // test different strides
     for &s1 in &[1, 2, -1, -2] {
         for &s2 in &[1, 2, -1, -2] {
             for &(m, k, n, q) in &sizes {
                 let mut a = range_mat64(m, k);
                 let mut answer = a.clone();
-                let cdim = if n == 1 {
-                    vec![q]
-                } else {
-                    vec![n, q]
-                };
+                let cdim = if n == 1 { vec![q] } else { vec![n, q] };
                 let cslice = if n == 1 {
                     vec![SliceOrIndex::from(..).step_by(s2)]
                 } else {
@@ -615,26 +604,28 @@ fn scaled_add_3() {
                     answerv += &(beta * &c);
                     av.scaled_add(beta, &c);
                 }
-                assert_close(a.view(), answer.view());
+                assert_relative_eq!(a, answer, epsilon = 1e-12, max_relative = 1e-7);
             }
         }
     }
 }
 
-
+#[cfg(feature = "approx")]
 #[test]
 fn gen_mat_mul() {
     let alpha = -2.3;
     let beta = 3.14;
-    let sizes = vec![(4, 4, 4), (8, 8, 8),
-                     (17, 15, 16),
-                     (4, 17, 3),
-                     (17, 3, 22),
-                     (19, 18, 2),
-                     (16, 17, 15),
-                     (15, 16, 17),
-                     (67, 63, 62),
-        ];
+    let sizes = vec![
+        (4, 4, 4),
+        (8, 8, 8),
+        (17, 15, 16),
+        (4, 17, 3),
+        (17, 3, 22),
+        (19, 18, 2),
+        (16, 17, 15),
+        (15, 16, 17),
+        (67, 63, 62),
+    ];
     // test different strides
     for &s1 in &[1, 2, -1, -2] {
         for &s2 in &[1, 2, -1, -2] {
@@ -654,14 +645,14 @@ fn gen_mat_mul() {
 
                     general_mat_mul(alpha, &a, &b, beta, &mut cv);
                 }
-                assert_close(c.view(), answer.view());
+                approx::assert_relative_eq!(c, answer, epsilon = 1e-12, max_relative = 1e-7);
             }
         }
     }
 }
 
-
 // Test y = A x where A is f-order
+#[cfg(feature = "approx")]
 #[test]
 fn gemm_64_1_f() {
     let a = range_mat64(64, 64).reversed_axes();
@@ -671,22 +662,24 @@ fn gemm_64_1_f() {
     let mut y = range_mat64(m, 1);
     let answer = reference_mat_mul(&a, &x) + &y;
     general_mat_mul(1.0, &a, &x, 1.0, &mut y);
-    assert_close(y.view(), answer.view());
+    approx::assert_relative_eq!(y, answer, epsilon = 1e-12, max_relative = 1e-7);
 }
 
 #[test]
 fn gen_mat_mul_i32() {
     let alpha = -1;
     let beta = 2;
-    let sizes = vec![(4, 4, 4), (8, 8, 8),
-                     (17, 15, 16),
-                     (4, 17, 3),
-                     (17, 3, 22),
-                     (19, 18, 2),
-                     (16, 17, 15),
-                     (15, 16, 17),
-                     (67, 63, 62),
-        ];
+    let sizes = vec![
+        (4, 4, 4),
+        (8, 8, 8),
+        (17, 15, 16),
+        (4, 17, 3),
+        (17, 3, 22),
+        (19, 18, 2),
+        (16, 17, 15),
+        (15, 16, 17),
+        (67, 63, 62),
+    ];
     for &(m, k, n) in &sizes {
         let a = range_i32(m, k);
         let b = range_i32(k, n);
@@ -698,20 +691,41 @@ fn gen_mat_mul_i32() {
     }
 }
 
+#[cfg(feature = "approx")]
 #[test]
 fn gen_mat_vec_mul() {
+    use approx::assert_relative_eq;
+    use ndarray::linalg::general_mat_vec_mul;
+
+    // simple, slow, correct (hopefully) mat mul
+    fn reference_mat_vec_mul<A, S, S2>(
+        lhs: &ArrayBase<S, Ix2>,
+        rhs: &ArrayBase<S2, Ix1>,
+    ) -> Array1<A>
+    where
+        A: LinalgScalar,
+        S: Data<Elem = A>,
+        S2: Data<Elem = A>,
+    {
+        let ((m, _), k) = (lhs.dim(), rhs.dim());
+        reference_mat_mul(lhs, &rhs.to_owned().into_shape((k, 1)).unwrap())
+            .into_shape(m)
+            .unwrap()
+    }
+
     let alpha = -2.3;
     let beta = 3.14;
-    let sizes = vec![(4, 4),
-                     (8, 8),
-                     (17, 15),
-                     (4, 17),
-                     (17, 3),
-                     (19, 18),
-                     (16, 17),
-                     (15, 16),
-                     (67, 63),
-        ];
+    let sizes = vec![
+        (4, 4),
+        (8, 8),
+        (17, 15),
+        (4, 17),
+        (17, 3),
+        (19, 18),
+        (16, 17),
+        (15, 16),
+        (67, 63),
+    ];
     // test different strides
     for &s1 in &[1, 2, -1, -2] {
         for &s2 in &[1, 2, -1, -2] {
@@ -736,25 +750,45 @@ fn gen_mat_vec_mul() {
 
                         general_mat_vec_mul(alpha, &a, &b, beta, &mut cv);
                     }
-                    assert_close(c.view(), answer.view());
+                    assert_relative_eq!(c, answer, epsilon = 1e-12, max_relative = 1e-7);
                 }
             }
         }
     }
 }
 
+#[cfg(feature = "approx")]
 #[test]
 fn vec_mat_mul() {
-    let sizes = vec![(4, 4),
-                     (8, 8),
-                     (17, 15),
-                     (4, 17),
-                     (17, 3),
-                     (19, 18),
-                     (16, 17),
-                     (15, 16),
-                     (67, 63),
-        ];
+    use approx::assert_relative_eq;
+
+    // simple, slow, correct (hopefully) mat mul
+    fn reference_vec_mat_mul<A, S, S2>(
+        lhs: &ArrayBase<S, Ix1>,
+        rhs: &ArrayBase<S2, Ix2>,
+    ) -> Array1<A>
+    where
+        A: LinalgScalar,
+        S: Data<Elem = A>,
+        S2: Data<Elem = A>,
+    {
+        let (m, (_, n)) = (lhs.dim(), rhs.dim());
+        reference_mat_mul(&lhs.to_owned().into_shape((1, m)).unwrap(), rhs)
+            .into_shape(n)
+            .unwrap()
+    }
+
+    let sizes = vec![
+        (4, 4),
+        (8, 8),
+        (17, 15),
+        (4, 17),
+        (17, 3),
+        (19, 18),
+        (16, 17),
+        (15, 16),
+        (67, 63),
+    ];
     // test different strides
     for &s1 in &[1, 2, -1, -2] {
         for &s2 in &[1, 2, -1, -2] {
@@ -778,7 +812,7 @@ fn vec_mat_mul() {
 
                         c.slice_mut(s![..;s2]).assign(&a.dot(&b));
                     }
-                    assert_close(c.view(), answer.view());
+                    assert_relative_eq!(c, answer, epsilon = 1e-12, max_relative = 1e-7);
                 }
             }
         }
